#!/bin/sh
#
# Usage: usage.sh <path>
####################################################

P="$1"

FIRST=`echo $P | awk -F '/' '{print $1"/"$2"/"$3"/"$4}'`
BAD=`echo $P | grep \\\.`

if test ! -z "$BAD" || test "$FIRST" != "/backtrace/var/coronerd" && "$FIRST" != "/var/coronerd"; then
	echo "error: bad path" 1>&2
	exit 1
fi

if test -z "$P" || test ! -d "$P"; then
	echo "error: first argument must be a path" 1>&2
	exit 1
fi

P=`realpath "$P"`
TYPE=""
NAME=`basename "$P"`

# First, determine the type of the directory we are looking at.
if test -d "${P}/_objects" && test -d "${P}/database" &&
   test -d "${P}/crdb"; then
	TYPE="project"
elif test -f "${P}/_metadata"; then
	if test -d "${P}/_license"; then
		TYPE="instance"
	else
		TYPE="universe"
	fi
fi

if test -z "$TYPE"; then
	echo "error: directory must be instance, universe or project" 1>&2
	exit 1
fi

C="du --files0-from=- -c -BM"
N="tr -dc '0-9'"

# Objects
OBJECTS=`find "${P}" \( -name 'tx.*.dmp.zst' -o -name 'tx.*.dmp' -o -name 'tx.*.json.zst' -o -name 'tx.*.json' -o -name 'tx.*.btt.zst' -o -name 'tx.*.btt' -o -name 'tx.*.plcrash.zst' -o -name 'tx.*.plcrash' \) -print0 | $C | tail -1 | $N`

# Attachments
ATTACHMENTS=`find "${P}" -name 'tx*blob:_attachment-*' -print0 | $C | tail -1 | $N`

# Symbols
SYMBOLS=`find "${P}" \( -name '*.symb' -o -name '*.symb.zst' -o -name '*.sourcemap.zst' -o -name '*.sourcemap' -o -name '*.proguard.zst' -o -name '*.proguard' -o -name '*.symbols' \) -print0 | $C | tail -1 | $N`

# Database
DATABASE=`find "${P}" -type d \( -name crdb -o -name issues \) -print0 | $C | tail -1 | $N`

# Total
TOTAL=`du -BM -s "${P}" | tail -1 | awk '{print $1}' | $N`

T_C=`expr $OBJECTS + $ATTACHMENTS + $SYMBOLS + $DATABASE`

if test -z "$OUTPUT_HEADER"; then
	printf "# %8s %40.40s %8s %8s %8s %8s %8s ($T_C)\n" "Type" "Name" "Objects" "Attach" "Symbols" "Database" "Total" 1>&2
fi

# Replace spaces with underscore.
NAME=`echo $NAME | sed 's/&/\&amp;/g; s/ /%20/g; s/</\&lt;/g; s/>/\&gt;/g; s/"/\&quot;/g; s/'"'"'/\&#39;/g'`
printf "  %8s %40.40s %8d %8d %8d %8d %8d\n" "$TYPE" "$NAME" "$OBJECTS" "$ATTACHMENTS" "$SYMBOLS" "$DATABASE" "$TOTAL"

export OUTPUT_HEADER="0"

if test "$TYPE" != "project"; then
	find "${P}" -mindepth 1 -maxdepth 1 -type d ! -iname '_*' ! -iname 'users' -exec $0 {} \;
fi
