"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var axios_1 = tslib_1.__importDefault(require("axios"));
var events_1 = require("events");
var form_data_1 = tslib_1.__importDefault(require("form-data"));
var fs = tslib_1.__importStar(require("fs"));
var json_stringify_safe_1 = tslib_1.__importDefault(require("json-stringify-safe"));
var path = tslib_1.__importStar(require("path"));
var backtraceResult_1 = require("./model/backtraceResult");
var BacktraceApi = /** @class */ (function (_super) {
    tslib_1.__extends(BacktraceApi, _super);
    function BacktraceApi(backtraceUri, timeout) {
        var _this = _super.call(this) || this;
        _this.backtraceUri = backtraceUri;
        _this.timeout = timeout;
        return _this;
    }
    BacktraceApi.prototype.send = function (report) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var data, formData, result, err, err_1;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, report.toJson()];
                    case 1:
                        data = _a.sent();
                        this.emit('before-data-send', report, data);
                        return [4 /*yield*/, this.getFormData(report, data)];
                    case 2:
                        formData = _a.sent();
                        _a.label = 3;
                    case 3:
                        _a.trys.push([3, 5, , 6]);
                        return [4 /*yield*/, axios_1.default.post(this.backtraceUri, formData, {
                                timeout: this.timeout,
                                headers: {
                                    'Content-Type': "multipart/form-data; boundary=" + formData.getBoundary(),
                                },
                            })];
                    case 4:
                        result = _a.sent();
                        if (result.status !== 200) {
                            err = new Error("Invalid attempt to submit error to Backtrace. Result: " + result);
                            return [2 /*return*/, backtraceResult_1.BacktraceResult.OnError(report, err)];
                        }
                        return [2 /*return*/, backtraceResult_1.BacktraceResult.Ok(report, result.data)];
                    case 5:
                        err_1 = _a.sent();
                        return [2 /*return*/, backtraceResult_1.BacktraceResult.OnError(report, err_1)];
                    case 6: return [2 /*return*/];
                }
            });
        });
    };
    BacktraceApi.prototype.getFormData = function (report, data) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var formData, json, attachments;
            return tslib_1.__generator(this, function (_a) {
                formData = new form_data_1.default();
                json = json_stringify_safe_1.default(data);
                formData.append('upload_file', json, 'upload_file.json');
                attachments = report.getAttachments();
                if (attachments instanceof Array) {
                    attachments.forEach(function (filePath) {
                        var result = fs.existsSync(filePath);
                        if (!result) {
                            return;
                        }
                        var name = path.basename(filePath);
                        formData.append("attachment_" + name, fs.createReadStream(filePath), name);
                    });
                }
                return [2 /*return*/, formData];
            });
        });
    };
    return BacktraceApi;
}(events_1.EventEmitter));
exports.BacktraceApi = BacktraceApi;
//# sourceMappingURL=backtraceApi.js.map