"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var fs = tslib_1.__importStar(require("fs"));
var path = tslib_1.__importStar(require("path"));
/**
 * Read module dependencies
 */
function readModuleDependencies(modulePath) {
    if (!modulePath) {
        return undefined;
    }
    var packageJson = require(modulePath);
    return {
        requestedVersions: packageJson.dependencies || packageJson.peerDependencies || packageJson.devDependencies,
        devDependencies: packageJson.devDependencies,
    };
}
exports.readModuleDependencies = readModuleDependencies;
function readModule(root, depth) {
    if (depth === void 0) { depth = 10; }
    if (depth < 0) {
        return readLibModule();
    }
    if (!root) {
        root = process.cwd();
    }
    // solve problem when root module doesn't exists
    if (!fs.existsSync(root)) {
        return readParentDir(root, depth);
    }
    var filePath = fs.lstatSync(root).isFile();
    if (filePath) {
        root = path.dirname(root);
    }
    var modulePredictedPath = path.join(root, 'package.json');
    var exists = fs.existsSync(modulePredictedPath);
    if (!exists) {
        return readParentDir(root, depth);
    }
    try {
        return [require(modulePredictedPath), modulePredictedPath];
    }
    catch (err) {
        // possible solution that can goes wrong:
        // * not enough privileges to read module,
        // * cannot import module
        // * module not exists
        return readParentDir(root, depth);
    }
}
exports.readModule = readModule;
function readLibModule() {
    console.warn('reading Backtrace module - cannot find correct module');
    return require('../../package.json');
}
function readParentDir(root, depth) {
    var parent = path.join(root, '..');
    return readModule(parent, --depth);
}
//# sourceMappingURL=moduleResolver.js.map