import { BacktraceClient } from './backtraceClient';
import { BacktraceClientOptions, IBacktraceClientOptions } from './model/backtraceClientOptions';
import * as btReport from './model/backtraceReport';
import { BacktraceResult } from './model/backtraceResult';
export { IBacktraceData } from './model/backtraceData';
export { BacktraceClient } from './backtraceClient';
export { BacktraceReport as BtReport } from './model/backtraceReport';
export { BacktraceClientOptions, IBacktraceClientOptions } from './model/backtraceClientOptions';
/**
 * Initalize Backtrace Client and Backtrace node integration
 * @param configuration Bcktrace configuration
 */
export declare function initialize(configuration: BacktraceClientOptions | IBacktraceClientOptions): BacktraceClient;
/**
 * Returns used BacktraceClient
 */
export declare function getBacktraceClient(): BacktraceClient;
export declare function use(client: BacktraceClient): void;
/**
 * Send report asynchronously to Backtrace
 * @param arg report payload
 * @param arg2 attributes
 * @param arg3 file attachments paths
 */
export declare function report(arg: Error | string | object | ((data?: Error) => void), arg2?: object | undefined, arg3?: string[] | ((data?: Error) => void), arg4?: string[]): Promise<BacktraceResult>;
/**
 * Send report asynchronously to Backtrace
 * @param payload report payload
 * @param reportAttributes attributes
 * @param fileAttachments file attachments paths
 */
export declare function reportAsync(payload: Error | string, reportAttributes?: object | undefined, fileAttachments?: string[]): Promise<BacktraceResult>;
/**
 * Send Backtrace report to Backtrace asynchronously.
 * @param data Backtrace report
 */
export declare function sendAsync(data: btReport.BacktraceReport): Promise<BacktraceResult>;
/**
 * Memorize selected values from application.
 * Memorized attributes will be available in your next Backtrace report.
 * Memorized attributes will be only available for one report.
 * @param key attribute key
 * @param value attribute value
 */
export declare function memorize(key: string, value: any): void;
/**
 * Clear all saved attributes
 */
export declare function clearMemorizedAttributes(): void;
/**
 * Returns all memorized attributes without clearing them.
 */
export declare function checkMemorizedAttributes(): object;
/**
 * Set symbolication info
 */
export declare function setSymbolication(): void;
/**
 * Add symbolication map to each report.
 * @param symbolicationMap
 */
export declare function setSymbolicationMap(symbolicationMap: Array<{
    file: string;
    uuid: string;
}>): void;
/**
 * Send backtrace report to Backtrace
 */
export declare function sendReport(data: btReport.BacktraceReport, callback?: (err?: Error) => void): BacktraceResult;
/**
 * Send report synchronosuly to Backtrace
 * @param error report payload
 * @param reportAttributes attributes
 * @param attachments file attachments paths
 */
export declare function reportSync(data: Error | string, attributes?: object | undefined, attachments?: string[]): BacktraceResult;
/**
 * Generaten BacktraceReport with default configuration
 */
export declare function createReport(): btReport.BacktraceReport;
/**
 * Generaten BacktraceReport with default configuration
 */
export declare function BacktraceReport(): btReport.BacktraceReport;
export declare function errorHandlerMiddleware(err: Error, req: any, resp: any, next: any): void;
