"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var backtraceClient_1 = require("./backtraceClient");
var backtraceClient;
var backtraceClient_2 = require("./backtraceClient");
exports.BacktraceClient = backtraceClient_2.BacktraceClient;
var backtraceReport_1 = require("./model/backtraceReport");
exports.BtReport = backtraceReport_1.BacktraceReport;
var backtraceClientOptions_1 = require("./model/backtraceClientOptions");
exports.BacktraceClientOptions = backtraceClientOptions_1.BacktraceClientOptions;
/**
 * Initalize Backtrace Client and Backtrace node integration
 * @param configuration Bcktrace configuration
 */
function initialize(configuration) {
    backtraceClient = new backtraceClient_1.BacktraceClient(configuration);
    return backtraceClient;
}
exports.initialize = initialize;
/**
 * Returns used BacktraceClient
 */
function getBacktraceClient() {
    return backtraceClient;
}
exports.getBacktraceClient = getBacktraceClient;
function use(client) {
    backtraceClient = client;
}
exports.use = use;
/**
 * Send report asynchronously to Backtrace
 * @param arg report payload
 * @param arg2 attributes
 * @param arg3 file attachments paths
 */
function report(arg, arg2, 
// tslint:disable-next-line: ban-types
arg3, arg4) {
    if (arg2 === void 0) { arg2 = {}; }
    if (arg3 === void 0) { arg3 = []; }
    if (arg4 === void 0) { arg4 = []; }
    return tslib_1.__awaiter(this, void 0, void 0, function () {
        var data, attachments, callback, result;
        return tslib_1.__generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    if (!backtraceClient) {
                        throw new Error('Must call initialize method first');
                    }
                    data = '';
                    if (arg instanceof Error || typeof arg === 'string') {
                        data = arg;
                    }
                    if (typeof arg === 'object' && arg2 === {}) {
                        arg2 = arg;
                    }
                    attachments = arg3 instanceof Function ? arg4 : arg3;
                    callback = arg instanceof Function ? arg : arg3;
                    return [4 /*yield*/, backtraceClient.reportAsync(data, arg2, attachments)];
                case 1:
                    result = _a.sent();
                    if (callback && callback instanceof Function) {
                        callback(result.Error);
                    }
                    return [2 /*return*/, result];
            }
        });
    });
}
exports.report = report;
/**
 * Send report asynchronously to Backtrace
 * @param payload report payload
 * @param reportAttributes attributes
 * @param fileAttachments file attachments paths
 */
function reportAsync(payload, reportAttributes, fileAttachments) {
    if (reportAttributes === void 0) { reportAttributes = {}; }
    if (fileAttachments === void 0) { fileAttachments = []; }
    return tslib_1.__awaiter(this, void 0, void 0, function () {
        return tslib_1.__generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    if (!backtraceClient) {
                        throw new Error('Must call initialize method first');
                    }
                    return [4 /*yield*/, backtraceClient.reportAsync(payload, reportAttributes, fileAttachments)];
                case 1: return [2 /*return*/, _a.sent()];
            }
        });
    });
}
exports.reportAsync = reportAsync;
/**
 * Send Backtrace report to Backtrace asynchronously.
 * @param data Backtrace report
 */
function sendAsync(data) {
    return tslib_1.__awaiter(this, void 0, void 0, function () {
        return tslib_1.__generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    if (!backtraceClient) {
                        throw new Error('Must call initialize method first');
                    }
                    return [4 /*yield*/, backtraceClient.sendAsync(data)];
                case 1: return [2 /*return*/, _a.sent()];
            }
        });
    });
}
exports.sendAsync = sendAsync;
/**
 * Memorize selected values from application.
 * Memorized attributes will be available in your next Backtrace report.
 * Memorized attributes will be only available for one report.
 * @param key attribute key
 * @param value attribute value
 */
function memorize(key, value) {
    if (!backtraceClient) {
        throw new Error('Must call initialize method first');
    }
    backtraceClient.memorize(key, value);
}
exports.memorize = memorize;
/**
 * Clear all saved attributes
 */
function clearMemorizedAttributes() {
    if (!backtraceClient) {
        throw new Error('Must call initialize method first');
    }
    backtraceClient.clearMemorizedAttributes();
}
exports.clearMemorizedAttributes = clearMemorizedAttributes;
/**
 * Returns all memorized attributes without clearing them.
 */
function checkMemorizedAttributes() {
    var _a;
    return (_a = backtraceClient) === null || _a === void 0 ? void 0 : _a.checkMemorizedAttributes();
}
exports.checkMemorizedAttributes = checkMemorizedAttributes;
/**
 * Set symbolication info
 */
function setSymbolication() {
    if (!backtraceClient) {
        throw new Error('Must call initialize method first');
    }
    backtraceClient.setSymbolication();
}
exports.setSymbolication = setSymbolication;
/**
 * Add symbolication map to each report.
 * @param symbolicationMap
 */
function setSymbolicationMap(symbolicationMap) {
    if (!backtraceClient) {
        throw new Error('Must call initialize method first');
    }
    backtraceClient.setSymbolicationMap(symbolicationMap);
}
exports.setSymbolicationMap = setSymbolicationMap;
/**
 * Send backtrace report to Backtrace
 */
function sendReport(data, callback) {
    if (!backtraceClient) {
        throw new Error('Must call initialize method first');
    }
    return backtraceClient.sendReport(data, callback);
}
exports.sendReport = sendReport;
/**
 * Send report synchronosuly to Backtrace
 * @param error report payload
 * @param reportAttributes attributes
 * @param attachments file attachments paths
 */
function reportSync(data, attributes, attachments) {
    if (attributes === void 0) { attributes = {}; }
    if (attachments === void 0) { attachments = []; }
    if (!backtraceClient) {
        throw new Error('Must call initialize method first');
    }
    return backtraceClient.reportSync(data, attributes, attachments);
}
exports.reportSync = reportSync;
/**
 * Generaten BacktraceReport with default configuration
 */
function createReport() {
    return BacktraceReport();
}
exports.createReport = createReport;
/**
 * Generaten BacktraceReport with default configuration
 */
function BacktraceReport() {
    if (!backtraceClient) {
        throw new Error('Must call initialize method first');
    }
    var backtraceReport = backtraceClient.createReport('');
    backtraceReport.send = function (callback) {
        backtraceClient.sendReport(backtraceReport, callback);
    };
    backtraceReport.sendSync = function (callback) {
        backtraceClient.sendReport(backtraceReport, callback);
    };
    return backtraceReport;
}
exports.BacktraceReport = BacktraceReport;
function errorHandlerMiddleware(err, req, resp, next) {
    if (!backtraceClient) {
        throw new Error('Must call initialize method first');
    }
    backtraceClient.reportSync(err, tslib_1.__assign(tslib_1.__assign({}, req), resp));
    next(err);
}
exports.errorHandlerMiddleware = errorHandlerMiddleware;
//# sourceMappingURL=index.js.map