import { BacktraceReport } from './backtraceReport';
export declare enum BacktraceResultStatus {
    /**
     * Set when sampling method decide to skip report
     */
    SamplingHit = 0,
    /**
     * Set when client limit is reached
     */
    LimitReached = 1,
    /**
     * Set when error occurs while sending diagnostic data
     */
    ServerError = 2,
    /**
     * Set when data were send to API
     */
    Ok = 4,
    /**
     * Data were send to API and waiting for server result
     */
    InProcessing = 8
}
/**
 * Send method result
 */
export declare class BacktraceResult {
    private report;
    private message;
    private status;
    private err?;
    static Processing(report: BacktraceReport): BacktraceResult;
    static Ok(report: BacktraceReport, result: string): BacktraceResult;
    /**
     * Set result when client rate limit reached
     * @param report Executed report
     * @returns  BacktraceResult with limit reached information
     */
    static OnLimitReached(report: BacktraceReport): BacktraceResult;
    static OnSamplingHit(report: BacktraceReport): BacktraceResult;
    /**
     * Set result when error occurs while sending data to API
     * @param report Executed report
     * @param error Error
     * @returns  BacktraceResult with exception information
     */
    static OnError(report: BacktraceReport, err: Error): BacktraceResult;
    private readonly _objectId;
    private readonly _rxId;
    get ObjectId(): string;
    get Report(): BacktraceReport;
    get Message(): string;
    get Error(): Error | undefined;
    get Status(): BacktraceResultStatus;
    private constructor();
}
