"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
var axios_1 = __importDefault(require("axios"));
var body_parser_1 = __importDefault(require("body-parser"));
var crypto_1 = require("crypto");
var https_1 = __importDefault(require("https"));
var apiError_1 = require("../model/apiError");
var ProjectValidation = __importStar(require("../service/projectValidation"));
/**
 * Identity manager
 */
var IdentityManager = /** @class */ (function () {
    function IdentityManager(descr) {
        this.descr = descr;
        /*
         * Stores opaque objects per authenticated remote coronerd.
         * Available for consumers to use as they see fit.
         */
        this.authenticated = {};
        this.logger = undefined;
        this.httpsAgent = this.descr.isOnPremise
            ? new https_1.default.Agent({ rejectUnauthorized: false })
            : https_1.default.globalAgent;
    }
    IdentityManager.prototype.setLogger = function (logger) {
        if (!logger || !logger.log) {
            throw new TypeError('unusable logger passed in');
        }
        this.logger = logger;
    };
    /*
     * Generate a middleware handler for Express.js, using options to hook the
     * service up.
     *
     * @param opts ICoronerRequestOption - coronerd url callback if desired
     */
    IdentityManager.prototype.serviceRequest = function (opts) {
        var _this = this;
        /*
         * Handle <prefix>/service requests.  These require action QS.  Other QS
         * are defined per action.
         */
        return [
            body_parser_1.default.json(),
            function (request, response, next) {
                if (!request.body) {
                    throw new apiError_1.ApiError('Missing body', 400);
                }
                var action = request.body.action;
                if (!action) {
                    throw new apiError_1.ApiError('Missing action parameter', 400);
                }
                switch (action) {
                    case 'register':
                        var _a = request.body, url = _a.url, nonce = _a.nonce, hmac = _a.hmac;
                        if (!url || !nonce || !hmac) {
                            throw new apiError_1.ApiError("missing parameters\n              " + (!url ? 'url' : '') + "\n              " + (!nonce ? 'nonce' : '') + "\n              " + (!hmac ? 'hmac' : ''), 400);
                        }
                        if (!_this.checkHmac(_this.descr.secret, nonce, hmac)) {
                            throw new apiError_1.ApiError('Unauthorized', 403);
                        }
                        /* Track authenticated URLs. */
                        if (!_this.authenticated[url]) {
                            _this.authenticated[url] = {};
                        }
                        if (opts && opts.coronerdCallback) {
                            opts.coronerdCallback(url);
                        }
                        // Ignore project validation in service layer handshake.
                        ProjectValidation.setValid(response);
                        response.json({
                            status: 'ok',
                        });
                        break;
                    default:
                        throw new apiError_1.ApiError('invalid action', 400);
                }
            },
        ];
    };
    /**
     * Generate a middleware for Express.js, which authenticates a request given
     * its URL and session token, provided by an earlier callback in
     * req.coronerAuth.
     * Upon success, the configuration data will be in req.coronerAuth.cfg.
     */
    IdentityManager.prototype.authenticateRequest = function () {
        var _this = this;
        return function (request, response, next) { return __awaiter(_this, void 0, void 0, function () {
            var _a, url, token, serverConfiguration, authRequest, err_1;
            return __generator(this, function (_b) {
                switch (_b.label) {
                    case 0:
                        _a = this.readCoronerAuthInfo(request), url = _a.url, token = _a.token;
                        if (!token || !url) {
                            next(new apiError_1.ApiError(request.ip + ": Missing internal params = token or url", 403));
                            return [2 /*return*/];
                        }
                        _b.label = 1;
                    case 1:
                        _b.trys.push([1, 3, , 4]);
                        return [4 /*yield*/, this.authenticate(url, token)];
                    case 2:
                        serverConfiguration = _b.sent();
                        if (!serverConfiguration || !serverConfiguration.user) {
                            this.bypassProjectAccessCheck(response);
                            next(new apiError_1.ApiError('Auth failure', 403));
                            return [2 /*return*/];
                        }
                        authRequest = request.coronerAuth;
                        if (authRequest) {
                            authRequest.cfg = serverConfiguration;
                        }
                        next();
                        return [3 /*break*/, 4];
                    case 3:
                        err_1 = _b.sent();
                        this.bypassProjectAccessCheck(response);
                        if (err_1.isAxiosError && err_1.response.status) {
                            next(new apiError_1.ApiError(err_1.message, err_1.response.status));
                        }
                        else {
                            next(new apiError_1.ApiError(err_1.message, 500));
                        }
                        return [3 /*break*/, 4];
                    case 4: return [2 /*return*/];
                }
            });
        }); };
    };
    /**
     * This assumes route has already passed authentication through authenticateRequest
     * and uses /api/config result from that to check projects.
     */
    IdentityManager.prototype.checkProjectAccess = function (request, response, projects) {
        var _a, _b, _c, _d, _e, _f, _g;
        var authRequest = request;
        var universe = (_c = (_b = (_a = authRequest) === null || _a === void 0 ? void 0 : _a.coronerAuth) === null || _b === void 0 ? void 0 : _b.cfg) === null || _c === void 0 ? void 0 : _c.universe;
        var projectsExt = universe && ((_g = (_f = (_e = (_d = authRequest) === null || _d === void 0 ? void 0 : _d.coronerAuth) === null || _e === void 0 ? void 0 : _e.cfg) === null || _f === void 0 ? void 0 : _f.universes[universe.name]) === null || _g === void 0 ? void 0 : _g.projects_ext);
        if (!projectsExt) {
            this.log('debug', "missing projects_ext");
            return false; // invalidated because flag is never set to true
        }
        return ProjectValidation.check(response, projectsExt, projects);
    };
    /**
     * Unconditionally set projectValidated flag to true.
     * Allows service routes that do not need project validation to procceed with
     * typical Express send method.
     */
    IdentityManager.prototype.bypassProjectAccessCheck = function (response) {
        ProjectValidation.setValid(response);
    };
    /**
     * Use coronerd auth method to validate url and session token provided by user
     * For Express apps, consider using the authenticateRequest middleware instead
     * @param url coronerd url
     * @param token coronerd token
     */
    IdentityManager.prototype.authenticate = function (url, token) {
        return __awaiter(this, void 0, void 0, function () {
            var prefix, result, coronerd_nonce, coronerd_hmac;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (!token || !url) {
                            throw new apiError_1.ApiError('missing url/token', 400);
                        }
                        prefix = url.endsWith('/') ? '' : '/';
                        return [4 /*yield*/, axios_1.default.get("" + url + prefix + "api/config", {
                                httpsAgent: this.httpsAgent,
                                headers: {
                                    'X-Service-Name': this.descr.name,
                                    'X-Coroner-Token': this.sanitizeHeader(token),
                                    'X-Service-HMAC': this.generateHmac(this.descr.secret, token),
                                },
                            })];
                    case 1:
                        result = _a.sent();
                        if (result.status !== 200) {
                            throw new apiError_1.ApiError("Invalid server response code " + result.status, result.status);
                        }
                        coronerd_nonce = result.headers['x-service-nonce'];
                        coronerd_hmac = result.headers['x-service-hmac'];
                        if (!coronerd_nonce || !coronerd_hmac) {
                            throw new apiError_1.ApiError('Missing response parameters - coronerd_nonce/hmac', 400);
                        }
                        if (!this.checkHmac(this.descr.secret, coronerd_nonce, coronerd_hmac)) {
                            throw new apiError_1.ApiError('Invalid server generated HMAC', 400);
                        }
                        return [2 /*return*/, result.data];
                }
            });
        });
    };
    /**
     * Login to a coronerd.
     * @param Coronerd base URL to login to
     */
    IdentityManager.prototype.loginCoronerd = function (coronerdUrl) {
        return __awaiter(this, void 0, void 0, function () {
            var prefix, protocol, requestUrl, nonce, hmac, result, err_2;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (!coronerdUrl) {
                            throw new Error('CoronerdUrl is undefined or empty string');
                        }
                        prefix = coronerdUrl.endsWith('/') ? '' : '/';
                        protocol = coronerdUrl.startsWith('http') ? '' : 'http://';
                        requestUrl = "" + protocol + coronerdUrl + prefix + "api/login";
                        nonce = crypto_1.randomBytes(32).toString('hex');
                        hmac = this.generateHmac(this.descr.secret, nonce);
                        _a.label = 1;
                    case 1:
                        _a.trys.push([1, 3, , 4]);
                        return [4 /*yield*/, axios_1.default.post(requestUrl, null, {
                                httpsAgent: this.httpsAgent,
                                headers: {
                                    'X-Service-Name': this.descr.name,
                                    'X-Service-Nonce': nonce,
                                    'X-Service-HMAC': hmac,
                                },
                            })];
                    case 2:
                        result = _a.sent();
                        return [2 /*return*/, result.status === 200 ? result.data : undefined];
                    case 3:
                        err_2 = _a.sent();
                        this.log('error', "Cannot login to coronerd. Reason " + err_2.message);
                        return [2 /*return*/, undefined];
                    case 4: return [2 /*return*/];
                }
            });
        });
    };
    /**
     * @deprecated: these methods will be removed in new backtrace-service  update
     * Check if authtoken is a valid token generated by coronerd
     * @param universeUrl url to coronerd universe
     * @param authToken user token
     */
    IdentityManager.prototype.isValidToken = function (universeUrl, authToken) {
        return __awaiter(this, void 0, void 0, function () {
            var result;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.getConfiguration(universeUrl, authToken)];
                    case 1:
                        result = _a.sent();
                        return [2 /*return*/, result !== undefined];
                }
            });
        });
    };
    /**
     * @deprecated: these methods will be removed in new backtrace-service  update
     * Get Backtrace configuration
     * @param universeUrl url to coronerd universe
     * @param authToken user token
     */
    IdentityManager.prototype.getConfiguration = function (universeUrl, authToken) {
        return __awaiter(this, void 0, void 0, function () {
            var prefix, protocol, requestUrl, result, err_3;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (!authToken || !universeUrl) {
                            return [2 /*return*/, undefined];
                        }
                        prefix = universeUrl.endsWith('/') ? '' : '/';
                        protocol = universeUrl.startsWith('http') ? '' : 'http://';
                        requestUrl = "" + protocol + universeUrl + prefix + "api/config";
                        _a.label = 1;
                    case 1:
                        _a.trys.push([1, 3, , 4]);
                        return [4 /*yield*/, axios_1.default.get(requestUrl, {
                                headers: {
                                    'X-Coroner-Token': authToken,
                                },
                            })];
                    case 2:
                        result = _a.sent();
                        return [2 /*return*/, result.status === 200 ? result.data : undefined];
                    case 3:
                        err_3 = _a.sent();
                        return [2 /*return*/, undefined];
                    case 4: return [2 /*return*/];
                }
            });
        });
    };
    IdentityManager.prototype.generateHmac = function (secret, nonce) {
        if (!secret || !nonce) {
            return '';
        }
        return crypto_1.createHmac('sha256', secret).update(nonce).digest('hex');
    };
    IdentityManager.prototype.checkHmac = function (secret, nonce, proposedHMac) {
        var hmac = this.generateHmac(secret, nonce);
        if (!hmac) {
            return false;
        }
        return hmac === proposedHMac;
    };
    IdentityManager.prototype.sanitizeHeader = function (value) {
        if (!value) {
            return '';
        }
        return value.replace(/[^a-z0-9A-Z?!]/gi, '');
    };
    IdentityManager.prototype.log = function (level, msg) {
        if (this.logger) {
            this.logger.log(level, msg);
        }
    };
    IdentityManager.prototype.readCoronerAuthInfo = function (request) {
        var data = request.coronerAuth;
        if (!data) {
            data = {
                token: this.sanitizeHeader(request.get('X-Coroner-Token')),
                url: request.get('X-Coroner-Location') ||
                    request.get('origin'),
            };
            if (data.url && !data.url.startsWith('http')) {
                data.url = "http://" + data.url;
            }
            request.coronerAuth = data;
        }
        return data;
    };
    return IdentityManager;
}());
exports.IdentityManager = IdentityManager;
// vim:set ts=2 sw=2 et:
//# sourceMappingURL=identity.js.map