"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
var backtrace_node_1 = require("backtrace-node");
var winston_1 = __importStar(require("winston"));
var Logger = /** @class */ (function () {
    function Logger(logLevel) {
        if (logLevel === void 0) { logLevel = 'debug'; }
        this.logLevel = logLevel;
        // transportLevel definition that Logger class respect:
        // #define SD_EMERG   "<0>"  /* system is unusable */
        // #define SD_ALERT   "<1>"  /* action must be taken immediately */
        // #define SD_CRIT    "<2>"  /* critical conditions */
        // #define SD_ERR     "<3>"  /* error conditions */
        // #define SD_WARNING "<4>"  /* warning conditions */
        // #define SD_NOTICE  "<5>"  /* normal but significant condition */
        // #define SD_INFO    "<6>"  /* informational */
        // #define SD_DEBUG   "<7>"  /* debug-level messages */
        this.transportLevel = {
            error: 3,
            warn: 4,
            info: 5,
            verbose: 6,
            debug: 7,
            silly: 6,
        };
        winston_1.configure({
            transports: new winston_1.transports.Console({
                level: logLevel,
                silent: process.env.NODE_ENV === 'test',
                format: winston_1.format.combine(winston_1.format.colorize(), winston_1.format.simple()),
            }),
        });
        Logger._logger = this;
    }
    Logger.getLogger = function () {
        if (!Logger._logger) {
            this._logger = new Logger();
            this._logger.warn("Creating new default logger instance");
        }
        return this._logger;
    };
    Logger.initializeBacktrace = function (opts) {
        if (!opts || !opts.endpoint) {
            return;
        }
        var logger = this.getLogger();
        logger.debug('Initializing backtrace-node integration');
        var client = backtrace_node_1.initialize(opts);
        if (!client) {
            logger.debug('Cannot initialize Backtrace integration. Please verify Backtrace url');
        }
    };
    Logger.prototype.trace = function (message) {
        winston_1.default.verbose(this.formatScope(message));
    };
    Logger.prototype.info = function (message) {
        winston_1.default.info(this.formatScope(message));
    };
    Logger.prototype.warn = function (message) {
        winston_1.default.warn(this.formatScope(message));
    };
    Logger.prototype.debug = function (message) {
        winston_1.default.debug(this.formatScope(message));
    };
    Logger.prototype.error = function (data, attributes) {
        var exception = data instanceof Error ? data : new Error(data);
        var client = backtrace_node_1.getBacktraceClient();
        if (client) {
            client.reportSync(exception, attributes);
        }
        winston_1.default.error(this.formatScope(exception.message));
    };
    Logger.prototype.getTransportLevel = function () {
        return this.transportLevel[this.logLevel] || this.transportLevel['debug'];
    };
    Logger.prototype.formatScope = function (msg) {
        return ("<" + this.getTransportLevel() + ">" +
            (this.logLevel + ":[" + new Date().toLocaleString() + "] " + msg));
    };
    return Logger;
}());
exports.Logger = Logger;
//# sourceMappingURL=logger.js.map