"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var fs_1 = __importDefault(require("fs"));
var path_1 = __importDefault(require("path"));
var __1 = require("..");
/**
 * Backtrace service configuration. Class responsible for managing configuration paths/parsing them.
 */
var ServiceConfiguration = /** @class */ (function () {
    function ServiceConfiguration() {
        this._logger = __1.Logger.getLogger();
    }
    /**
     * @deprecated Please use instead getProjectSettingsPath. This method is deprecated because
     * of the way how we want to read configuration files. This method will return only one
     * path to service configuration, that might be wrong for complex environment when user want to
     * override default configuration.
     *
     * Select default path for service configuraiton path in different environment
     * By default Backtrace expect to run service from root service directory
     * by using npm commands. In environment different than production
     * @ServiceConfiguration class will read configuration file from root dir.
     * Otherwise from default Backtrace configuration directory
     */
    ServiceConfiguration.prototype.getProjectSettingsPath = function (serviceName) {
        switch (process.env.NODE_ENV) {
            case 'production':
                return "/etc/backtrace/" + serviceName + "/" + serviceName + ".conf";
            default:
                return path_1.default.join(process.cwd(), serviceName + ".conf");
        }
    };
    /**
     * Select default paths for service configuraiton path in different environment
     * By default Backtrace expect to run service from root service directory
     * by using npm commands. In environment different than production
     * @ServiceConfiguration class will read configuration file from root dir.
     * Otherwise from default Backtrace configuration directory
     */
    ServiceConfiguration.prototype.getProjectSettingsPaths = function (serviceName) {
        switch (process.env.NODE_ENV) {
            case 'production':
                return [
                    "/etc/backtrace/" + serviceName + "/" + serviceName + ".conf",
                    path_1.default.join(process.cwd(), serviceName + ".conf"),
                ];
            default:
                return [path_1.default.join(process.cwd(), serviceName + ".conf")];
        }
    };
    /**
     * Loads service configuration file
     */
    ServiceConfiguration.prototype.loadConfiguration = function (configurationPaths) {
        if (!configurationPaths || configurationPaths.length === 0) {
            throw new Error("configurationPath is undefined");
        }
        for (var _i = 0, configurationPaths_1 = configurationPaths; _i < configurationPaths_1.length; _i++) {
            var configurationPath = configurationPaths_1[_i];
            // ignore undefined path or not existing configuration files.
            if (!configurationPath || !fs_1.default.existsSync(configurationPath)) {
                continue;
            }
            this._logger.info("Reading configuration file available " + configurationPath);
            var config = JSON.parse(fs_1.default.readFileSync(configurationPath, 'utf8'));
            config.isProduction = process.env.NODE_ENV === 'production';
            if (config.ssl) {
                if (config.ssl.key) {
                    config.ssl.key = fs_1.default.readFileSync(config.ssl.key);
                }
                if (config.ssl.cert) {
                    config.ssl.cert = fs_1.default.readFileSync(config.ssl.cert);
                }
            }
            return config;
        }
        throw new Error("Cannot find configuration file." +
            ("Backtrace-service configuration file paths: " + configurationPaths));
    };
    return ServiceConfiguration;
}());
exports.ServiceConfiguration = ServiceConfiguration;
//# sourceMappingURL=serviceConfiguration.js.map